@@ -1,76 +1,69 @@
 package Function::Parameters::Info;
 
 use v5.14.0;
+
 use warnings;
 
-our $VERSION = 1.0702_01;
+our $VERSION = '0.03';
 
 # If Moo isn't loaded yet but Moose is, avoid pulling in Moo and fall back to Moose
-my ($Moo, $meta_make_immutable);
+my $Moo;
 BEGIN {
-    if ($INC{'Moose.pm'} && !$INC{'Moo.pm'}) {
-        $Moo = 'Moose';
-        $meta_make_immutable = sub { $_[0]->meta->make_immutable };
-    } else {
-        require Moo;
-        $Moo = 'Moo';
-        $meta_make_immutable = sub {};
-    }
-    $Moo->import;
+	if ($INC{'Moose.pm'} && !$INC{'Moo.pm'}) {
+		$Moo = 'Moose';
+	} else {
+		require Moo;
+		$Moo = 'Moo';
+	}
+	$Moo->import;
 }
 
 {
-    package Function::Parameters::Param;
+	package Function::Parameters::Param;
 
-    BEGIN { $Moo->import; }
-    use overload
-        fallback => 1,
-        '""'     => sub { $_[0]->name },
-    ;
+	BEGIN { $Moo->import; }
+	use overload
+		fallback => 1,
+		'""' => sub { $_[0]->name },
+	;
 
-    has $_ => (is => 'ro') for qw(name type);
+	has $_ => (is => 'ro') for qw(name type);
 
-    __PACKAGE__->$meta_make_immutable;
+	__PACKAGE__->meta->make_immutable;
 }
 
-my @pn_ro = qw(
-    positional_required
-    positional_optional
-    named_required
-    named_optional
-);
+my @pn_ro = glob '{positional,named}_{required,optional}';
 
 for my $attr (qw[keyword invocant slurpy], map "_$_", @pn_ro) {
-    has $attr => (
-        is => 'ro',
-    );
+	has $attr => (
+		is => 'ro',
+	);
 }
 
-sub positional_required { @{$_[0]->_positional_required } }
-sub positional_optional { @{$_[0]->_positional_optional } }
-sub named_required      { @{$_[0]->_named_required } }
-sub named_optional      { @{$_[0]->_named_optional } }
+for my $gen (join "\n", map "sub $_ { \@{\$_[0]->_$_} }", @pn_ro) {
+	eval "$gen\n1" or die $@;
+}
 
 sub args_min {
-    my $self = shift;
-    my $r = 0;
-    $r++ if defined $self->invocant;
-    $r += $self->positional_required;
-    $r += $self->named_required * 2;
-    $r
+	my $self = shift;
+	my $r = 0;
+	$r++ if defined $self->invocant;
+	$r += $self->positional_required;
+	$r += $self->named_required * 2;
+	$r
 }
 
 sub args_max {
-    my $self = shift;
-    return 0 + 'Inf' if defined $self->slurpy || $self->named_required || $self->named_optional;
-    my $r = 0;
-    $r++ if defined $self->invocant;
-    $r += $self->positional_required;
-    $r += $self->positional_optional;
-    $r
+	my $self = shift;
+	return 0 + 'Inf' if defined $self->slurpy || $self->named_required || $self->named_optional;
+	my $r = 0;
+	$r++ if defined $self->invocant;
+	$r += $self->positional_required;
+	$r += $self->positional_optional;
+	$r
 }
 
-__PACKAGE__->$meta_make_immutable;
+__PACKAGE__->meta->make_immutable;
 
 'ok'
 
@@ -111,7 +104,7 @@ methods are available:
 =head2 $info->invocant
 
 Returns the name of the variable into which the first argument is
-L<C<shift>|perlfunc/shift ARRAY>ed automatically, or C<undef> if no such thing
+L<C<shift>|perlfunc/shift>ed automatically, or C<undef> if no such thing
 exists. This will usually return C<'$self'> for methods.
 
 =head2 $info->positional_required
